@@ -1,4 +1,86 @@
 
+#' @title Show one or more views of the given meshes in rgl windows.
+#'
+#' @param views list of strings. Valid entries include: 'si': single interactive view. 'sd_<angle>': single view from angle <angle>. The <angle> part must be one of the strings returned by \code{\link[fsbrain]{get.view.angle.names}}. Example: 'sd_caudal'. 'sr': single rotating view. 't4': tiled view showing the brain from 4 angles. 't9': tiled view showing the brain from 9 angles.
+#'
+#' @param coloredmeshes list of coloredmesh or renderable. A coloredmesh is a named list as returned by the coloredmesh.from.* functions. It has the entries 'mesh' of type tmesh3d, a 'col', which is a color specification for such a mesh.
+#'
+#' @param rgloptions option list passed to \code{\link[rgl]{par3d}}. Example: \code{rgloptions = list("windowRect"=c(50,50,1000,1000))}
+#'
+#' @param rglactions named list. A list in which the names are from a set of pre-defined actions. The values can be used to specify parameters for the action.
+#'
+#' @param style character string, a rendering style, e.g., 'default', 'shiny' or 'semitransparent'.
+#'
+#' @param draw_colorbar logical, whether to draw a colorbar. WARNING: The colorbar is drawn to a subplot, and this only works if there is enough space for it. You will have to increase the plot size using the 'rlgoptions' parameter for the colorbar to show up. Defaults to FALSE. See \code{\link[fsbrain]{coloredmesh.plot.colorbar.separate}} for an alternative.
+#'
+#' @return list of coloredmeshes. The coloredmeshes used for the visualization.
+#'
+#' @seealso \code{\link[fsbrain]{get.view.angle.names}}
+#'
+#' @export
+brainviews <- function(views, coloredmeshes, rgloptions = list(), rglactions = list(), style="default", draw_colorbar = FALSE) {
+
+    # Wrap a single instance into a list if needed
+    if(fsbrain.renderable(coloredmeshes)) {
+        print("Wrapping single renderable instance into list.");
+        coloredmeshes = list(coloredmeshes);
+    }
+
+    if(length(views)) {
+        for(view in views) {
+            if(view == "t4") {
+                invisible(brainview.t4(coloredmeshes, rgloptions = rgloptions, rglactions = rglactions, draw_colorbar = draw_colorbar, style = style));
+            } else if(view == "t9") {
+                invisible(brainview.t9(coloredmeshes, rgloptions = rgloptions, rglactions = rglactions, draw_colorbar = draw_colorbar, style = style));
+            } else if(view == "si") {
+                invisible(brainview.si(coloredmeshes, rgloptions = rgloptions, rglactions = rglactions, draw_colorbar = draw_colorbar, style = style));
+            } else if(view == "sr") {
+                invisible(brainview.sr(coloredmeshes, rgloptions = rgloptions, rglactions = rglactions, draw_colorbar = draw_colorbar, style = style));
+            } else if(startsWith(view, "sd_")) {
+                angle = substr(view, 4, nchar(view));
+                invisible(brainview.sd(coloredmeshes, angle, rgloptions = rgloptions, rglactions = rglactions, draw_colorbar = draw_colorbar, style = style));
+            } else {
+                stop(sprintf("Invalid view '%s'. Valid ones include 'si', 'sr', 'sd_<angle>', 't4' and 't9'.\n", view));
+            }
+        }
+    }
+    return(invisible(coloredmeshes));
+}
+
+
+#' @title Get list of valid view angle names.
+#'
+#' @description The returned strings are used as constants to identify a view of type `sd_<angle>`. They can be used to construct entries for the parameter `views` of functions like \code{\link[fsbrain]{vis.subject.morph.native}}, or directly as parameter 'view_angles' for functions like \code{\link[fsbrain]{vislayout.from.coloredmeshes}}.
+#'
+#' @param add_sd_prefix logical, whether the prefix 'sd_' should be added to the string. This will construct full view names. If set to false, only the substring after the prefix 'sd_' will be returned. This is used internally only and should not be needed in general.
+#'
+#' @param angle_set string, which view subset to return. Available subsets are: 'all' (or alias 't9'): for all 9 angles. 't4': for the t4 views. 'medial': the 2 medial views, one for each hemi. 'lateral': the 2 lateral views, one for each hemi. 'lh': medial and laterial for the left hemisphere. 'rh': medial and laterial for the right hemisphere.
+#'
+#' @return vector of character strings, all valid view angle strings.
+#' @export
+get.view.angle.names <- function(add_sd_prefix=TRUE, angle_set="all") {
+    if(angle_set == "all" || angle_set == "t9") {
+        angles = c('lateral_lh', 'dorsal', 'lateral_rh', 'medial_lh', 'ventral', 'medial_rh', 'rostral', 'caudal');
+    } else if(angle_set == "t4") {
+        angles = c('lateral_lh', 'lateral_rh', 'medial_lh', 'medial_rh');
+    } else if(angle_set == "medial") {
+        angles = c('medial_lh', 'medial_rh');
+    } else if(angle_set == "lateral") {
+        angles = c('lateral_lh', 'lateral_rh');
+    } else if(angle_set == "lh") {
+        angles = c('lateral_lh', 'medial_lh');
+    }  else if(angle_set == "rh") {
+        angles = c('lateral_rh', 'medial_rh');
+    } else {
+        stop(sprintf("Invalid 'angle_set' parameter: '%s'.\n", angle_set));
+    }
+
+    if(add_sd_prefix) {
+        angles = paste("sd_", angles, sep="");
+    }
+    return(angles);
+}
+
 
 #' @title Visualize a list of colored meshes from a single viewpoint, interactively.
 #'
@@ -6,7 +88,7 @@
 #'
 #' @param background string, background color passed to \code{\link[rgl]{bg3d}}.
 #'
-#' @param skip_all_na logical, whether to skip (i.e., not render) meshes in the list that have the property 'morph_data_was_all_na' set to TRUE. Defaults to TRUE. Practically, this means that a hemisphere for which the data was not given is not rendered, instead of being rendered in a single color.
+#' @param skip_all_na logical, whether to skip (i.e., not render) meshes in the list that have the property 'render' set to FALSE. Defaults to TRUE. Practically, this means that a hemisphere for which the data was not given is not rendered, instead of being rendered in a single color.
 #'
 #' @param style, a named list of style parameters or a string specifying an available style by name (e.g., 'shiny'). Defaults to 'default', the default style.
 #'
@@ -25,7 +107,7 @@ brainview.si <- function(coloredmeshes, background="white", skip_all_na=TRUE, st
     if(!is.list(coloredmeshes)) {
         stop("Parameter 'coloredmeshes' must be a list.");
     }
-    invisible(vis.coloredmeshes(coloredmeshes, rgloptions = rgloptions, rglactions = rglactions, draw_colorbar = draw_colorbar));
+    return(invisible(vis.coloredmeshes(coloredmeshes, rgloptions = rgloptions, rglactions = rglactions, draw_colorbar = draw_colorbar, style = style)));
 }
 
 
@@ -35,7 +117,7 @@ brainview.si <- function(coloredmeshes, background="white", skip_all_na=TRUE, st
 #'
 #' @param background string, background color passed to \code{\link[rgl]{bg3d}}.
 #'
-#' @param skip_all_na logical, whether to skip (i.e., not render) meshes in the list that have the property 'morph_data_was_all_na' set to TRUE. Defaults to TRUE. Practically, this means that a hemisphere for which the data was not given is not rendered, instead of being rendered in a single color.
+#' @param skip_all_na logical, whether to skip (i.e., not render) meshes in the list that have the property 'render' set to FALSE. Defaults to TRUE. Practically, this means that a hemisphere for which the data was not given is not rendered, instead of being rendered in a single color.
 #'
 #' @param style, a named list of style parameters or a string specifying an available style by name (e.g., 'shiny'). Defaults to 'default', the default style.
 #'
@@ -64,7 +146,34 @@ brainview.sr <- function(coloredmeshes, background="white", skip_all_na=TRUE, st
     if(!is.list(coloredmeshes)) {
         stop("Parameter 'coloredmeshes' must be a list.");
     }
-    invisible(vis.coloredmeshes.rotating(coloredmeshes, x=x, y=y, z=z, rpm=rpm, duration=duration, rgloptions = rgloptions, rglactions = rglactions));
+    return(invisible(vis.coloredmeshes.rotating(coloredmeshes, x=x, y=y, z=z, rpm=rpm, duration=duration, rgloptions = rgloptions, rglactions = rglactions, style = style)));
+}
+
+
+#' @keywords internal
+get.sorted.cmeshes <- function(coloredmeshes) {
+    if("lh" %in% names(coloredmeshes) | "rh" %in% names(coloredmeshes)) {
+        # Use the new style of coloredmeshes: a list with entries 'lh' and 'rh', each of which contains a single cmesh
+        if("lh" %in% names(coloredmeshes)) {
+            lh_meshes = list(coloredmeshes$lh);
+        } else {
+            lh_meshes = NULL;
+        }
+
+        if("rh" %in% names(coloredmeshes)) {
+            rh_meshes = list(coloredmeshes$rh);
+        } else {
+            rh_meshes = NULL;
+        }
+    } else {
+        # This is the old style of passing the list: unsorted with unmerged colormaps. We need to fiddle with the data.
+        # Some functions still use this, but they will all be reworked in the future. Once done, this part can be removed.
+        warning("The old style of passing coloredmeshes should not be in use anymore.");
+        hemi_sorted_cmeshes = sort.coloredmeshes.by.hemi(coloredmeshes);
+        lh_meshes = hemi_sorted_cmeshes$lh;
+        rh_meshes = hemi_sorted_cmeshes$rh;
+    }
+    return(list("lh"=lh_meshes, "rh"=rh_meshes));
 }
 
 
@@ -74,7 +183,7 @@ brainview.sr <- function(coloredmeshes, background="white", skip_all_na=TRUE, st
 #'
 #' @param background string, background color passed to \code{\link[rgl]{bg3d}}.
 #'
-#' @param skip_all_na logical, whether to skip (i.e., not render) meshes in the list that have the property 'morph_data_was_all_na' set to TRUE. Defaults to TRUE. Practically, this means that a hemisphere for which the data was not given is not rendered, instead of being rendered in a single color.
+#' @param skip_all_na logical, whether to skip (i.e., not render) meshes in the list that have the property 'render' set to FALSE. Defaults to TRUE. Practically, this means that a hemisphere for which the data was not given is not rendered, instead of being rendered in a single color.
 #'
 #' @param style, a named list of style parameters or a string specifying an available style by name (e.g., 'shiny'). Defaults to 'default', the default style.
 #'
@@ -96,26 +205,42 @@ brainview.t4 <- function(coloredmeshes, background="white", skip_all_na=TRUE, st
         stop("Parameter 'coloredmeshes' must be a list.");
     }
 
-    layout_dim_x = 2;
-    layout_dim_y = 2;
-    num_views = layout_dim_x * layout_dim_y;
+    horizontal = FALSE;
+    if(draw_colorbar == TRUE) {
+        draw_colorbar = "vertical";
+    }
 
-    coloredmeshes = unify.coloredmeshes.colormaps(coloredmeshes);
+    if(draw_colorbar == "vertical") {
+        layout_mat = matrix(c(1, 2, 5, 3, 4, 6), ncol=3, byrow = T);
+        layout_column_widths = c(3L, 3L, 1L);
+        layout_row_heights = rep(1L, nrow(layout_mat));
+    } else if(draw_colorbar == "horizontal") {
+        horizontal = TRUE;
+        layout_mat = matrix(seq.int(6), ncol=2, byrow = T);
+        layout_row_heights = c(3L, 3L, 1L);
+        layout_column_widths = rep(1L, ncol(layout_mat));
+    } else if(draw_colorbar == FALSE) {
+        # assume FALSE
+        layout_mat = matrix(seq.int(4), ncol=2, byrow = T);
+        layout_column_widths = rep(1L, ncol(layout_mat));
+        layout_row_heights = rep(1L, nrow(layout_mat));
+    } else {
+        stop("Invalid setting for 'draw_colorbar'. Use a logical value or one of 'horizontal' or 'vertical'.");
+    }
 
-    hemi_sorted_cmeshes = sort.coloredmeshes.by.hemi(coloredmeshes);
-
-    lh_meshes = hemi_sorted_cmeshes$lh;
-    rh_meshes = hemi_sorted_cmeshes$rh;
 
+    sorted_meshes = get.sorted.cmeshes(coloredmeshes);
+    lh_meshes = sorted_meshes$lh;
+    rh_meshes = sorted_meshes$rh;
 
     rgl::open3d();
     do.call(rgl::par3d, rgloptions);
     Sys.sleep(1);
     rgl::bg3d(background);
-    rgl::mfrow3d(layout_dim_x, layout_dim_y);
+    rgl::layout3d(layout_mat, widths=layout_column_widths, height=layout_row_heights);
 
     # Create the upper left view: draw only the left hemi, from the left
-    rgl::next3d();
+    rgl::next3d(reuse=TRUE);
     vis.rotated.coloredmeshes(lh_meshes, pi/2, 1, 0, 0, style=style);
     rgl::rgl.viewpoint(-90, 0, fov=0, interactive=FALSE);
     if(draw_labels) {
@@ -123,7 +248,7 @@ brainview.t4 <- function(coloredmeshes, background="white", skip_all_na=TRUE, st
     }
 
     # Create the upper right view
-    rgl::next3d();
+    rgl::next3d(reuse=FALSE);
     vis.rotated.coloredmeshes(rh_meshes, pi/2, 1, 0, 0, style=style);
     rgl::rgl.viewpoint(90, 0, fov=0, interactive=FALSE);
     if(draw_labels) {
@@ -132,8 +257,8 @@ brainview.t4 <- function(coloredmeshes, background="white", skip_all_na=TRUE, st
 
 
     # Create the lower left view
-    rgl::next3d();
-    vis.rotated.coloredmeshes(lh_meshes, pi/2, 1, 0, 0, style=style, draw_colorbar=(draw_colorbar && (length(rh_meshes)==0)));
+    rgl::next3d(reuse=FALSE);
+    vis.rotated.coloredmeshes(lh_meshes, pi/2, 1, 0, 0, style=style);
     rgl::rgl.viewpoint(90, 0, fov=0, interactive=FALSE);
     if(draw_labels) {
         rgl::text3d(0,label_shift_y,0,"medial lh");
@@ -141,16 +266,23 @@ brainview.t4 <- function(coloredmeshes, background="white", skip_all_na=TRUE, st
 
 
     # Create the lower right view
-    rgl::next3d();
-    vis.rotated.coloredmeshes(rh_meshes, pi/2, 1, 0, 0, style=style, draw_colorbar=draw_colorbar);
+    rgl::next3d(reuse=FALSE);
+    vis.rotated.coloredmeshes(rh_meshes, pi/2, 1, 0, 0, style=style);
     rgl::rgl.viewpoint(-90, 0, fov=0, interactive=FALSE);
     if(draw_labels) {
         rgl::text3d(0,label_shift_y,0,"medial rh");
     }
 
+    if(is.character(draw_colorbar)) {
+        rgl::next3d(reuse=FALSE);
+        draw.colorbar(coloredmeshes, horizontal=horizontal);
+        rgl::next3d(reuse=FALSE);
+        draw.colorbar(coloredmeshes, horizontal=horizontal);
+    }
+
 
     perform.rglactions(rglactions);
-    invisible(coloredmeshes);
+    return(invisible(coloredmeshes));
 }
 
 
@@ -166,6 +298,9 @@ brainview.t4 <- function(coloredmeshes, background="white", skip_all_na=TRUE, st
 #' @importFrom rgl rgl.snapshot
 perform.rglactions <- function(rglactions, at_index=NULL) {
     if(is.list(rglactions)) {
+        if("text" %in% names(rglactions)) {
+            do.call(rgl::text3d, rglactions$text);
+        }
         if("snapshot_png" %in% names(rglactions)) {
             if(length(rglactions$snapshot_png) == 1 || is.null(at_index)) {
                 output_image = path.expand(rglactions$snapshot_png);
@@ -203,7 +338,7 @@ rglactions.has.key <- function(rglactions, key) {
 #'
 #' @param background string, background color passed to \code{\link[rgl]{bg3d}}.
 #'
-#' @param skip_all_na logical, whether to skip (i.e., not render) meshes in the list that have the property 'morph_data_was_all_na' set to TRUE. Defaults to TRUE. Practically, this means that a hemisphere for which the data was not given is not rendered, instead of being rendered in a single color.
+#' @param skip_all_na logical, whether to skip (i.e., not render) meshes in the list that have the property 'render' set to FALSE. Defaults to TRUE. Practically, this means that a hemisphere for which the data was not given is not rendered, instead of being rendered in a single color.
 #'
 #' @param style, a named list of style parameters or a string specifying an available style by name (e.g., 'shiny'). Defaults to 'default', the default style.
 #'
@@ -232,25 +367,45 @@ brainview.t9 <- function(coloredmeshes, background="white", skip_all_na=TRUE, st
     layout_dim_y = 3;
     num_views = layout_dim_x * layout_dim_y;
 
-    coloredmeshes = unify.coloredmeshes.colormaps(coloredmeshes);
+    sorted_meshes = get.sorted.cmeshes(coloredmeshes);
+    lh_meshes = sorted_meshes$lh;
+    rh_meshes = sorted_meshes$rh;
 
-    hemi_sorted_cmeshes = sort.coloredmeshes.by.hemi(coloredmeshes);
-    lh_meshes = hemi_sorted_cmeshes$lh;
-    rh_meshes = hemi_sorted_cmeshes$rh;
+    horizontal = FALSE;
+    if(draw_colorbar == TRUE) {
+        draw_colorbar = "vertical";
+    }
 
+    if(draw_colorbar == "vertical") {
+        layout_mat = matrix(c(1, 2, 3, 10, 4, 5, 6, 11, 7, 8, 9, 12), ncol=4, byrow = T);
+        layout_column_widths = c(3L, 3L, 3L, 1L);
+        layout_row_heights = rep(1L, nrow(layout_mat));
+    } else if(draw_colorbar == "horizontal") {
+        horizontal = TRUE;
+        layout_mat = matrix(seq.int(12), ncol=3, byrow = T);
+        layout_row_heights = c(3L, 3L, 3L, 1L);
+        layout_column_widths = rep(1L, ncol(layout_mat));
+    } else if(draw_colorbar == FALSE) {
+        # assume FALSE
+        layout_mat = matrix(seq.int(9), ncol=3, byrow = T);
+        layout_column_widths = rep(1L, ncol(layout_mat));
+        layout_row_heights = rep(1L, nrow(layout_mat));
+    } else {
+        stop("Invalid setting for 'draw_colorbar'. Use a logical value or one of 'horizontal' or 'vertical'.");
+    }
 
     rgl::open3d();
     do.call(rgl::par3d, rgloptions);
     Sys.sleep(1);
     rgl::bg3d(background);
-    rgl::mfrow3d(layout_dim_x, layout_dim_y, sharedMouse = TRUE);
+    rgl::layout3d(layout_mat, widths=layout_column_widths, height=layout_row_heights);
 
 
     #  ------------------ Row 1 --------------------
 
 
     # Create the upper left view: draw only the left hemi, from the left
-    rgl::next3d();
+    rgl::next3d(reuse=TRUE);
     vis.rotated.coloredmeshes(lh_meshes, pi/2, 1, 0, 0, style=style);
     rgl::rgl.viewpoint(-90, 0, fov=0, interactive=FALSE);
     if(draw_labels) {
@@ -258,7 +413,7 @@ brainview.t9 <- function(coloredmeshes, background="white", skip_all_na=TRUE, st
     }
 
     # Create the upper central view: draw both hemis from above (top view)
-    rgl::next3d();
+    rgl::next3d(reuse=FALSE);
     vis.rotated.coloredmeshes(coloredmeshes, 0, 1, 0, 0, style=style);
     rgl::rgl.viewpoint(0, 0, fov=0, interactive=FALSE);
     if(draw_labels) {
@@ -266,7 +421,7 @@ brainview.t9 <- function(coloredmeshes, background="white", skip_all_na=TRUE, st
     }
 
     # Create the upper right view
-    rgl::next3d();
+    rgl::next3d(reuse=FALSE);
     vis.rotated.coloredmeshes(rh_meshes, pi/2, 1, 0, 0, style=style);
     rgl::rgl.viewpoint(90, 0, fov=0, interactive=FALSE);
     if(draw_labels) {
@@ -278,7 +433,7 @@ brainview.t9 <- function(coloredmeshes, background="white", skip_all_na=TRUE, st
 
 
     # Create the 2nd row left view
-    rgl::next3d();
+    rgl::next3d(reuse=FALSE);
     vis.rotated.coloredmeshes(lh_meshes, pi/2, 1, 0, 0, style=style);
     rgl::rgl.viewpoint(90, 0, fov=0, interactive=FALSE);
     if(draw_labels) {
@@ -286,8 +441,8 @@ brainview.t9 <- function(coloredmeshes, background="white", skip_all_na=TRUE, st
     }
 
     # Create the 2nd row central view: draw both hemis from below (bottom view)
-    rgl::next3d();
-    vis.rotated.coloredmeshes(coloredmeshes, pi, 1, 0, 0, style=style, draw_colorbar = draw_colorbar);
+    rgl::next3d(reuse=FALSE);
+    vis.rotated.coloredmeshes(coloredmeshes, pi, 1, 0, 0, style=style);
     rgl::rgl.viewpoint(0, 0, fov=0, interactive=FALSE);
     if(draw_labels) {
         rgl::text3d(0,label_shift_y_ventral,0,"ventral");
@@ -295,7 +450,7 @@ brainview.t9 <- function(coloredmeshes, background="white", skip_all_na=TRUE, st
 
 
     # Create the 2nd row right view
-    rgl::next3d();
+    rgl::next3d(reuse=FALSE);
     vis.rotated.coloredmeshes(rh_meshes, pi/2, 1, 0, 0, style=style);
     rgl::rgl.viewpoint(-90, 0, fov=0, interactive=FALSE);
     if(draw_labels) {
@@ -307,7 +462,7 @@ brainview.t9 <- function(coloredmeshes, background="white", skip_all_na=TRUE, st
 
 
     # Create the bottom left view: draw only the left hemi, from the left
-    rgl::next3d();
+    rgl::next3d(reuse=FALSE);
     vis.rotated.coloredmeshes(coloredmeshes, pi/2, 1, 0, 0, style=style);
     rgl.viewpoint(0, 0, fov=0, interactive=FALSE);
     if(draw_labels) {
@@ -315,25 +470,28 @@ brainview.t9 <- function(coloredmeshes, background="white", skip_all_na=TRUE, st
     }
 
     # Create the bottom central view.
-    rgl::next3d();
+    rgl::next3d(reuse=FALSE);
     if(draw_labels) {
         rgl::text3d(0,label_shift_y,0,"(empty)");
-    } else {
-        # If we do not draw anything, the next3d() calls seems to get ignored,
-        #  and the next image gets places here instead of into the last field.
-        #  Therefore, we draw empty text for now.
-        rgl::text3d(0,label_shift_y,0,"");
-
     }
 
     # Create the bottom right view
-    rgl::next3d();
+    rgl::next3d(reuse=FALSE);
     vis.rotated.coloredmeshes(coloredmeshes, pi/2, 1, 0, 0, style=style);
     rgl::rgl.viewpoint(180, 0, fov=0, interactive=FALSE);
     if(draw_labels) {
         rgl::text3d(0,label_shift_y,0,"caudal");
     }
 
+    if(is.character(draw_colorbar)) {
+        rgl::next3d(reuse=FALSE);
+        draw.colorbar(coloredmeshes, horizontal=horizontal);
+        rgl::next3d(reuse=FALSE);
+        draw.colorbar(coloredmeshes, horizontal=horizontal);
+        rgl::next3d(reuse=FALSE);
+        draw.colorbar(coloredmeshes, horizontal=horizontal);
+    }
+
 
     perform.rglactions(rglactions);
     return(invisible(coloredmeshes));
@@ -348,7 +506,7 @@ brainview.t9 <- function(coloredmeshes, background="white", skip_all_na=TRUE, st
 #'
 #' @param background string, background color passed to \code{\link[rgl]{bg3d}}.
 #'
-#' @param skip_all_na logical, whether to skip (i.e., not render) meshes in the list that have the property 'morph_data_was_all_na' set to TRUE. Defaults to TRUE. Practically, this means that a hemisphere for which the data was not given is not rendered, instead of being rendered in a single color.
+#' @param skip_all_na logical, whether to skip (i.e., not render) meshes in the list that have the property 'render' set to FALSE. Defaults to TRUE. Practically, this means that a hemisphere for which the data was not given is not rendered, instead of being rendered in a single color.
 #'
 #' @param style, a named list of style parameters or a string specifying an available style by name (e.g., 'shiny'). Defaults to 'default', the default style.
 #'
@@ -367,11 +525,33 @@ brainview.sd <- function(coloredmeshes, view_angle, background="white", skip_all
         stop("Parameter 'coloredmeshes' must be a list.");
     }
 
-    coloredmeshes = unify.coloredmeshes.colormaps(coloredmeshes);
+    horizontal = FALSE;
+    if(draw_colorbar == TRUE) {
+        draw_colorbar = "vertical";
+    }
 
-    hemi_sorted_cmeshes = sort.coloredmeshes.by.hemi(coloredmeshes);
-    lh_meshes = hemi_sorted_cmeshes$lh;
-    rh_meshes = hemi_sorted_cmeshes$rh;
+    if(draw_colorbar == "vertical") {
+        layout_mat = matrix(c(1, 2), ncol=2, byrow = T);
+        layout_column_widths = c(3L, 1L);
+        layout_row_heights = rep(1L, nrow(layout_mat));
+    } else if(draw_colorbar == "horizontal") {
+        horizontal = TRUE;
+        layout_mat = matrix(c(1, 2), ncol=1, byrow = T);
+        layout_row_heights = c(3L, 1L);
+        layout_column_widths = rep(1L, ncol(layout_mat));
+    } else if(draw_colorbar == FALSE) {
+        # assume FALSE
+        layout_mat = NULL;
+        layout_column_widths = NULL;
+        layout_row_heights = NULL;
+    } else {
+        stop("Invalid setting for 'draw_colorbar'. Use a logical value or one of 'horizontal' or 'vertical'.");
+    }
+
+
+    sorted_meshes = get.sorted.cmeshes(coloredmeshes);
+    lh_meshes = sorted_meshes$lh;
+    rh_meshes = sorted_meshes$rh;
 
 
     rgl::open3d();
@@ -379,35 +559,45 @@ brainview.sd <- function(coloredmeshes, view_angle, background="white", skip_all
     Sys.sleep(1);
     rgl::bg3d(background);
 
+    if(is.character(draw_colorbar)) {
+        rgl::layout3d(layout_mat, widths=layout_column_widths, height=layout_row_heights);
+        rgl::next3d(reuse=TRUE);
+    }
+
 
     if(view_angle == "lateral_lh") {
-        vis.rotated.coloredmeshes(lh_meshes, pi/2, 1, 0, 0, style=style, draw_colorbar = draw_colorbar);
+        vis.rotated.coloredmeshes(lh_meshes, pi/2, 1, 0, 0, style=style);
         rgl::rgl.viewpoint(-90, 0, fov=0, interactive=FALSE);
     } else if (view_angle == "dorsal") {
-        vis.rotated.coloredmeshes(coloredmeshes, 0, 1, 0, 0, style=style, draw_colorbar = draw_colorbar);
+        vis.rotated.coloredmeshes(coloredmeshes, 0, 1, 0, 0, style=style);
         rgl::rgl.viewpoint(0, 0, fov=0, interactive=FALSE);
     } else if(view_angle == "lateral_rh") {
-        vis.rotated.coloredmeshes(rh_meshes, pi/2, 1, 0, 0, style=style, draw_colorbar = draw_colorbar);
+        vis.rotated.coloredmeshes(rh_meshes, pi/2, 1, 0, 0, style=style);
         rgl::rgl.viewpoint(90, 0, fov=0, interactive=FALSE);
     } else if(view_angle == "medial_lh") {
-        vis.rotated.coloredmeshes(lh_meshes, pi/2, 1, 0, 0, style=style, draw_colorbar = draw_colorbar);
+        vis.rotated.coloredmeshes(lh_meshes, pi/2, 1, 0, 0, style=style);
         rgl::rgl.viewpoint(90, 0, fov=0, interactive=FALSE);
     } else if(view_angle == "ventral") {
-        vis.rotated.coloredmeshes(coloredmeshes, pi, 1, 0, 0, style=style, draw_colorbar = draw_colorbar);
+        vis.rotated.coloredmeshes(coloredmeshes, pi, 1, 0, 0, style=style);
         rgl::rgl.viewpoint(0, 0, fov=0, interactive=FALSE);
     } else if(view_angle == "medial_rh") {
-        vis.rotated.coloredmeshes(rh_meshes, pi/2, 1, 0, 0, style=style, draw_colorbar = draw_colorbar);
+        vis.rotated.coloredmeshes(rh_meshes, pi/2, 1, 0, 0, style=style);
         rgl::rgl.viewpoint(-90, 0, fov=0, interactive=FALSE);
     } else if(view_angle == "rostral") {
-        vis.rotated.coloredmeshes(coloredmeshes, pi/2, 1, 0, 0, style=style, draw_colorbar = draw_colorbar);
+        vis.rotated.coloredmeshes(coloredmeshes, pi/2, 1, 0, 0, style=style);
         rgl.viewpoint(0, 0, fov=0, interactive=FALSE);
     } else if(view_angle == "caudal") {
-        vis.rotated.coloredmeshes(coloredmeshes, pi/2, 1, 0, 0, style=style, draw_colorbar = draw_colorbar);
+        vis.rotated.coloredmeshes(coloredmeshes, pi/2, 1, 0, 0, style=style);
         rgl::rgl.viewpoint(180, 0, fov=0, interactive=FALSE);
     } else {
         stop(sprintf("Invalid view_angle '%s'. Must be one of 'lateral_lh', 'dorsal', 'lateral_rh', 'medial_lh', 'ventral', 'medial_rh', 'rostral' or 'caudal'.\n", view_angle));
     }
 
+    if(is.character(draw_colorbar)) {
+        rgl::next3d(reuse=FALSE);
+        draw.colorbar(coloredmeshes, horizontal=horizontal);
+    }
+
     perform.rglactions(rglactions);
 
     return(invisible(coloredmeshes));
